"""
This file contains the multi runner class which can run jobs parallel using threads

TODO: implement
"""


class MultiRunner:
    """
    MultiRunner instance

    TODO: implement the multi runner (split the job queue and use n runner classes for that)
    """

    pass
